#include "gloobus-preview-window.h"



// MENU CALLBACK FUNCTIONS:
void blog(GtkWidget *object, gpointer *data)
{
	g_print("Blog\n");
}

void donate(GtkWidget *object, gpointer *data)
{
	g_print("Donate\n");
}

void prova(GtkWidget *widget, GdkEventExpose *event, gpointer prInstance)
{
	g_print("Prova\n");
}


// @param w window width
// @param h window height
// 
// Creates the window and defines the quit callback and the expose callback
// To add mouse and key events, <get_window_pointer> function should be used
// Draw a nice cairo background
// Draws a close button
void gloobusWin::create_window(int w, int h)
{
	//g_print ("\nCreating Window...");
	m_w = w;
	m_h = h;
	
	m_window = gtk_window_new ( GTK_WINDOW_TOPLEVEL );

	GdkScreen * screen = gdk_screen_get_default();
	gtk_widget_set_default_colormap ( gdk_screen_get_rgba_colormap (screen) );
	gtk_window_set_default_size 	( GTK_WINDOW(m_window), m_w, m_h);

	gtk_window_set_position			( GTK_WINDOW (m_window), GTK_WIN_POS_CENTER_ALWAYS);
	gtk_widget_set_app_paintable 	( m_window, 			 TRUE );
	gtk_window_set_decorated 		( GTK_WINDOW (m_window), FALSE);
	//gtk_window_set_skip_taskbar_hint( GTK_WINDOW (m_window), TRUE);			//Show in taskbar or not
	gtk_widget_realize 				( m_window );
	gdk_window_set_back_pixmap 		( m_window->window, NULL, FALSE);			//For the iMpg to not turn it black

	create_container();			// Creates a gtk_fixed container
	
	//Window destroy Signal
	g_signal_connect ( GTK_OBJECT(m_window), "destroy", G_CALLBACK(&gloobusWin::quit2) , this);
	g_signal_connect ( m_window, "expose-event", G_CALLBACK(&gloobusWin::trick), this);
	//g_signal_connect ( m_window, "expose-event", G_CALLBACK(prova), NULL);
}
// ============================ GET WINDOW POINTER ============================= //
// @return GtkWidget* A GtkWindow widget
//
//This functions returns the window so then can be added other callbacks
//suchas key events, mouse events and more
GtkWidget*  gloobusWin::get_window_pointer 	( void )
{
	return m_window;
}
// ========================== GET CONTAINER POINTER ============================= //
// @return GtkWidget*  A GtkFixed widget
//the container so there can be added anything
GtkWidget*  gloobusWin::get_container_pointer 	( void )
{
	return m_container;
}

// ================================== QUIT ====================================== //
// TODO: This functions should not be implemented by window class
void gloobusWin::quit()
{
	
	gtk_main_quit();
}

// =============================== WIN SHOW ALL =============================== //
void gloobusWin::show_all()
{
	gtk_widget_show		(m_window); 		//The last show called
	gtk_widget_show 	(m_container);
}
// ================================ SET TITLE ================================ //
void gloobusWin::set_title (string title )
{
	m_title = title;
	m_title = m_title.substr(m_title.rfind("/")+1,-1);
	
	//Trim to max width
	float 	pixels_per_char = 6;
	int 	max = m_title.length();
	int 	width = m_w - SHADOW_WIDTH*2 - 40;	/* -40 = close button, and options button*/
	
	g_debug("Chars %i - Width: %i => Space for = %f\n", max, width , width/max*pixels_per_char);
	if(width/max*pixels_per_char  < (int)m_title.length())
	{	
		m_title = m_title.substr(0,width/max*pixels_per_char );
		m_title = m_title + "...";	
	}
	gtk_window_set_title (GTK_WINDOW (m_window), m_title.c_str()); 
}
// ================================ SET SIZE ================================ //
void gloobusWin::set_size( int w, int h)
{
	m_w = w;
	m_h = h;
	gtk_window_resize(GTK_WINDOW (m_window),m_w,m_h);
}
// ================================== EXPOSE ================================== //
// Expose event, this function is called everytime the window is drawed or redrawed
// It clears everything and draw the backgroun, shadow and header
void gloobusWin::expose()
{
	//g_print("\nDrawing Window");

	cairo_t *cr;

	//Transparent background

	cr = gdk_cairo_create 	( m_window->window);
	cairo_set_source_rgba 	( cr, 0, 0, 0, 0);
	cairo_set_operator    	( cr, CAIRO_OPERATOR_SOURCE );
	cairo_paint	  			( cr );
	cairo_set_operator		( cr, CAIRO_OPERATOR_OVER);

	//Needs to be first cause it clears the inside!
	draw_shadow		(cr, SHADOW_WIDTH, SHADOW_WIDTH, m_w-SHADOW_WIDTH*2, m_h-SHADOW_WIDTH*2, WINDOW_RADIOUS);	
	
	draw_header		(cr, SHADOW_WIDTH, SHADOW_WIDTH,  m_w-SHADOW_WIDTH*2, HEADER_HEIGHT, HEADER_RADIOUS);

	draw_bottom		(cr, SHADOW_WIDTH, SHADOW_WIDTH+HEADER_HEIGHT, m_w-SHADOW_WIDTH*2, m_h-HEADER_HEIGHT-SHADOW_WIDTH*2, WINDOW_RADIOUS);
	
	draw_background	(cr, SHADOW_WIDTH, SHADOW_WIDTH+HEADER_HEIGHT, m_w-SHADOW_WIDTH*2, m_h-HEADER_HEIGHT-SHADOW_WIDTH*2, WINDOW_RADIOUS);
		
	draw_title		(cr, m_w);
	
	//cairo_destroy(cr)	;		//Do not uncomment this line, if its uncommented, the widgets in the window are not drawed
	
}


// ==================== DRAW HEADER======================== //
// @param cr Cairo context
// @param x y where the shadow starts
// @param w,h height and widht of the rectangle to draw the shadow
// @param r radious of the borders
void gloobusWin::draw_header	 	(cairo_t *cr, int x, int y, int w, int h, int r)
{
	cairo_save(cr);
	
	//Header
	cairo_set_source_rgba (cr, 0, 0, 0, 0.7);		//Header Color
	cairo_move_to(cr,x+r,y) ;                     	//Move to A
	cairo_line_to(cr,x+w-r,y);                    	//Straight line to B
	cairo_curve_to(cr,x+w-r,y,x+w,y,x+w,y+r);   //Curve to C
	cairo_line_to(cr,x+w,y+h);				//Straight line to D
	cairo_line_to(cr,x,y+h);				//Straight line to E
	cairo_line_to(cr,x,y+r);				//Straight line to F
	cairo_curve_to(cr,x,y+r,x,y,x+r,y);		//Curve to A
	cairo_fill (cr);
	
	//Stroke
	cairo_set_source_rgba (cr, 150, 150, 150, 0.8);
	cairo_set_line_width(cr , 0.4);
	cairo_move_to(cr, x, y + h);
	cairo_curve_to(cr,x,y+r,x,y,x+r,y);		//Curve to A
	//cairo_move_to(cr,x+r,y) ;                     	//Move to A
	cairo_line_to(cr,x+w-r,y);                    	//Straight line to B
	cairo_curve_to(cr,x+w-r,y,x+w,y,x+w,y+r);   //Curve to C
	cairo_line_to(cr,x+w,y+h);				//Straight line to D
	//cairo_line_to(cr,x,y+h);				//Straight line to E
	//cairo_line_to(cr,x,y+r);				//Straight line to F
	//cairo_curve_to(cr,x,y+r,x,y,x+r,y);		//Curve to A
	cairo_stroke (cr);
	
	cairo_restore(cr);
}
// ==================== DRAW BACKGROUND======================== //
// @param cr Cairo context
// @param x y where the shadow starts
// @param w,h height and widht of the rectangle to draw the shadow
// @param r radious of the borders
void gloobusWin::draw_background (cairo_t *cr, int x, int y, int w, int h, int r)
{
	cairo_save(cr);
	
	h = h -BOTTOM_HEIGHT;

	//Background
	cairo_set_source_rgba (cr, 0, 0, 0, 0.8);
	cairo_move_to	( cr,x,y );                     	// Move to A
	cairo_line_to	( cr,x+w,y );                    	//Straight line to B//
	cairo_line_to	( cr,x+w,y+h-r );                  	// Move to D
	//cairo_curve_to	( cr,x+w,y+h-r,x+w,y+h,x+w-r,y+h ); 	// Curve to E
	cairo_line_to	( cr,x,y+h-r );                    	// Line to F
	//cairo_curve_to	( cr,x+r,y+h,x,y+h,x,y+h-r );      	// Curve to G
	cairo_line_to	( cr,x,y )       ;               	// Line to H
	cairo_fill 		( cr );
	
	//Stroke
	cairo_set_source_rgba (cr, 150, 150, 150, 0.8);
	cairo_set_line_width(cr , 0.4);
	
	cairo_move_to	( cr,x,y+h-r );                    	// Line to F
	cairo_line_to	( cr,x,y);
	
	cairo_move_to	( cr,x+w,y);                    	// Line to F
	cairo_line_to	( cr,x+w,y+h-r);
	
	cairo_stroke 	( cr );

	cairo_restore(cr);
}

// ==================== DRAW BOTTOM======================== //
// @param cr Cairo context
// @param x y where the shadow starts
// @param w,h height and widht of the rectangle to draw the shadow
// @param r radious of the borders
void gloobusWin::draw_bottom (cairo_t *cr, int x, int y, int w, int h, int r)
{
	int h2 = h -BOTTOM_HEIGHT;
	
	cairo_save(cr);


	//Background
	cairo_set_source_rgba (cr, 0, 0, 0, 0.8);		//Header Color
	//cairo_set_source_rgba (cr, 0, 0, 0, 0.6);
	cairo_move_to	( cr,x,y+h2-r );                     	// Move to A
	cairo_line_to	( cr,x+w,y+h2-r );                    	//Straight line to B//
	cairo_line_to	( cr,x+w,y+h-r );                    	//Straight line to B//
	//cairo_line_to	( cr,x+w,y+h-r );                  	// Move to D
	cairo_curve_to	( cr,x+w,y+h-r,x+w,y+h,x+w-r,y+h ); // Curve to E
	cairo_line_to	( cr,x+r,y+h );                    	// Line to F
	cairo_curve_to	( cr,x+r,y+h,x,y+h,x,y+h-r );      	// Curve to G
	cairo_line_to	( cr,x,y+h2-r  ); 
	//cairo_line_to	( cr,x,y+h-r )       ;               	// Line to H
	cairo_fill 		( cr );
	
	//Stroke
	cairo_set_source_rgba (cr, 150, 150, 150, 0.8);
	cairo_set_line_width(cr , 0.4);
	
	//cairo_move_to	( cr,x,y+h2-r );                     	// Move to A
	cairo_move_to	( cr,x+w,y+h2-r );                    	//Straight line to B//
	cairo_line_to	( cr,x+w,y+h-r );                    	//Straight line to B//
	//cairo_line_to	( cr,x+w,y+h-r );                  	// Move to D
	cairo_curve_to	( cr,x+w,y+h-r,x+w,y+h,x+w-r,y+h ); // Curve to E
	cairo_line_to	( cr,x+r,y+h );                    	// Line to F
	cairo_curve_to	( cr,x+r,y+h,x,y+h,x,y+h-r );      	// Curve to G
	cairo_line_to	( cr,x,y+h2-r  ); 
	//cairo_line_to	( cr,x,y+h-r )       ;               	// Line to H
	
	cairo_stroke 	( cr );
	

	cairo_restore(cr);
}
// ==================== DRAW SHADOW =========================== //
// @param cr Cairo context
// @param x y where the shadow starts
// @param w,h height and widht of the rectangle to draw the shadow
// @param r radious of the borders
void gloobusWin::draw_shadow(cairo_t *cr, int x, int y, int w, int h, int r)
{
	cairo_save(cr);
	cairo_pattern_t* pattern;	

	//top left
	int r2 = r/2;
	pattern = cairo_pattern_create_radial	( x+r, y+r, r,x+r, y+r, r+SHADOW_R );
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, SHADOW_A );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, 0 );
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, x-SHADOW_R, y-SHADOW_R, r+SHADOW_R, r+SHADOW_R);
	cairo_fill 			(cr);

	//top
	pattern = cairo_pattern_create_linear	( 0.0, y-SHADOW_R, 0.0, y);
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, 0 );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, SHADOW_A);
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, x+r, y-SHADOW_R, w-r*2, SHADOW_R);
	cairo_fill 			(cr);

	//top Rigth
	pattern = cairo_pattern_create_radial	( w+x-r, y+r, r, w+x-r, y+r, r+SHADOW_R );
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, SHADOW_A );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, 0 );
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, w+x-r, y-SHADOW_R, r+SHADOW_R, r+SHADOW_R);
	cairo_fill 			(cr);
	
	//right
	pattern = cairo_pattern_create_linear	( w+x, 0.0, w+x+SHADOW_R, 0.0);
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, SHADOW_A );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, 0);
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, w+x, y+r, SHADOW_R, h-r*2);
	cairo_fill 			(cr);
	
		
	//Bottom Rigth
	pattern = cairo_pattern_create_radial	( w+x-r, h+y-r, r, w+x-r, h+y-r, r+SHADOW_R);
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, SHADOW_A );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, 0 );
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, w+x-r, h+y-r, r+SHADOW_R, r+SHADOW_R);
	cairo_fill 			(cr);
	
	//Bottom
	pattern = cairo_pattern_create_linear	( 0.0, h+y, 0.0, h+y+SHADOW_R);
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, SHADOW_A );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, 0);
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, x+r, h+y, w-r*2, SHADOW_R);
	cairo_fill 			(cr);

	//Bottom Left
	pattern = cairo_pattern_create_radial	( x+r, h+y-r, r, x+r, h+y-r, r+SHADOW_R);
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, SHADOW_A );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, 0 );
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, x-SHADOW_R, h+y-r, r+SHADOW_R, r+SHADOW_R);
	cairo_fill 			(cr);

	//Left
	pattern = cairo_pattern_create_linear	( x-SHADOW_R, 0.0, x, 0.0 );
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, 0 );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, SHADOW_A);
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, x-SHADOW_R, y+r, r+SHADOW_R, h-r*2);
	cairo_fill 			(cr);


	//Clear the area inside
	cairo_move_to		( cr, x+r, y);
	cairo_arc			( cr, x+w-r2, y+r2,   r2, PI*1.5, PI*2		);
	cairo_arc			( cr, x+w-r, y+h-r, r, 0,      PI*0.5	);
	cairo_arc			( cr, x+r,   y+h-r, r, PI*0.5, PI		);
	cairo_arc			( cr, x+r2,  y+r2,   r2, PI,     PI*1.5	);

	cairo_set_operator 	( cr, CAIRO_OPERATOR_CLEAR );
	cairo_fill			( cr );

	cairo_restore(cr);

}

// =================== DRAW TITLE ================================== //
void gloobusWin::draw_title (cairo_t *cr, int w)
{
	// WINDOW TITLE : Notice that the system must have "Lucida Grande" font

	cairo_save				(cr);
	cairo_select_font_face 	(cr, "Lucida Grande", CAIRO_FONT_SLANT_NORMAL,CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size 	(cr, 12.0);
	cairo_move_to 			(cr, w /2 - m_title.length()*3, SHADOW_WIDTH + HEADER_HEIGHT - 5);
	cairo_text_path 		(cr, m_title.c_str());
	cairo_set_source_rgba 	(cr, 1, 1, 1 ,1);
	cairo_fill_preserve		(cr);
	cairo_restore			(cr);
	
}

//==================== CREATE CONTAINER ============================ //
void gloobusWin::create_container	( void )
{
	m_container = gtk_fixed_new();								//Create a new container and
	gtk_container_add  (GTK_CONTAINER(m_window),m_container);	//Add it to the main window	
	gtk_widget_show   (m_container);							//This is where the plugins will put his things

}
// ==================== ADD CLOSE BUTTON =========================== //
void gloobusWin::add_close_button	( void )
{
	//g_print("\nAdd Close Button");
	string buttonstr = (string)PACKAGE_DATA_PATH + "/close_button.png";
	//g_debug("Close button: %s\n",buttonstr.c_str());
	GtkWidget *close_button = gtk_image_new_from_file(buttonstr.c_str());	//(string)PACKAGE_LIB_PATH
	GtkWidget *event_box    = gtk_event_box_new();
	
	gtk_event_box_set_visible_window(GTK_EVENT_BOX(event_box),false);
	gtk_container_add	(GTK_CONTAINER(event_box),close_button);
	gtk_fixed_put		(GTK_FIXED(m_container),event_box, SHADOW_WIDTH+8, SHADOW_WIDTH+2);
	gtk_widget_show_all	(GTK_WIDGET(event_box));
	
	g_signal_connect	(GTK_OBJECT(event_box),"button_press_event", G_CALLBACK(gtk_main_quit), NULL);

}
// ==================== ADD CLOSE BUTTON =========================== //
void gloobusWin::add_drag_button	( void )
{
	/*//g_print("\nAdd drag Button");
	GtkWidget *close_button = gtk_image_new_from_file("/usr/share/gloobus/close_button.png");
	GtkWidget *event_box    = gtk_event_box_new();
	
	gtk_event_box_set_visible_window(GTK_EVENT_BOX(event_box),true);
	gtk_container_add	(GTK_CONTAINER(event_box),close_button);
	gtk_fixed_put		(GTK_FIXED(m_container),event_box, SHADOW_WIDTH+50, SHADOW_WIDTH+2);
	gtk_widget_show_all	(GTK_WIDGET(event_box));
	
	g_signal_connect	(GTK_OBJECT(event_box),"button_press_event", G_CALLBACK(gtk_main_quit), NULL);
	*/

}
// ==================== ADD GLOOBUS BUTTON =========================== //
void gloobusWin::add_gloobus_button	( void )
{
	//g_print("\nAdd Gloobus Button");
	string buttonstr = (string)PACKAGE_DATA_PATH + "/gloobus_button.png";
	GtkWidget *gloobus_button = gtk_image_new_from_file(buttonstr.c_str());	//(string)PACKAGE_LIB_PATH
	GtkWidget *event_box    = gtk_event_box_new();
	
	gtk_event_box_set_visible_window(GTK_EVENT_BOX(event_box),false);
	gtk_container_add	(GTK_CONTAINER(event_box),gloobus_button);
	gtk_fixed_put		(GTK_FIXED(m_container),event_box, m_w-SHADOW_WIDTH - 23, SHADOW_WIDTH+2);
	gtk_widget_show_all	(GTK_WIDGET(event_box));
	
	g_signal_connect	(GTK_OBJECT(event_box),"button_press_event", G_CALLBACK(&gloobusWin::trick2), this);

}
// ==================== ADD COMMON BUTTONS =========================== //
void gloobusWin::add_common_buttons		( void )
{
	add_close_button();
	add_gloobus_button();
}
// ========================== SHOW MENU ============================== //
void gloobusWin::show_menu		( void )
{

	GtkWidget *menu 	= gtk_menu_new();
	GtkWidget *blog 	= gtk_menu_item_new_with_label("Blog");
	GtkWidget *donate 	= gtk_menu_item_new_with_label("Donate");
	GtkWidget *bug 		= gtk_menu_item_new_with_label("Report Bug");
	GtkWidget *quit 	= gtk_menu_item_new_with_label("Quit");
	
               
	gtk_menu_shell_append  (GTK_MENU_SHELL(menu),blog);
	gtk_menu_shell_append  (GTK_MENU_SHELL(menu),donate);
	gtk_menu_shell_append  (GTK_MENU_SHELL(menu),bug);
	gtk_menu_shell_append  (GTK_MENU_SHELL(menu),quit);

	gtk_signal_connect(GTK_OBJECT(blog),   "activate", G_CALLBACK(&gloobusWin::trick3),  this);
	gtk_signal_connect(GTK_OBJECT(donate), "activate", G_CALLBACK(&gloobusWin::trick3),  this);
	gtk_signal_connect(GTK_OBJECT(bug),    "activate", G_CALLBACK(&gloobusWin::trick3),  this);
	gtk_signal_connect(GTK_OBJECT(quit),   "activate", G_CALLBACK(gtk_main_quit), 	NULL);
	
	gtk_widget_show_all(GTK_WIDGET(menu));
	
	gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,1,gtk_get_current_event_time());
}

// ==================== LAUNCH MENU ACTION =========================== //
void gloobusWin::launch_menu_action(gchar * label)
{
	string cmd;
	string lb = label;
	if(lb== "Report Bug") cmd = "gnome-open \"https://bugs.launchpad.net/gloobus/\"";
	if(lb == "Blog")	cmd = "gnome-open \"http://gloobus.wordpress.com\"" ;
	if(lb == "Donate") 	cmd = "gnome-open \"https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=guitarboy000%40gmail%2ecom&item_name=Gloobus%2c%20A%20Quicklook%20for%20linux&no_shipping=0&no_note=1&tax=0&currency_code=EUR&lc=US&bn=PP%2dDonationsBF&chars\"" ;

	
	g_spawn_command_line_async(cmd.c_str(), NULL);

}

// =================== GET WIDHT AND GET HEIGHT
int gloobusWin::get_width	( void ){ return m_w;}
int gloobusWin::get_height	( void ){ return m_h;}
